/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-26 18:28:34.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.PrintingProject;
import cn.com.qmth.examcloud.core.print.entity.ProjectStatistic;
import cn.com.qmth.examcloud.core.print.repository.ProjectStatisticRepository;
import cn.com.qmth.examcloud.core.print.service.CourseStatisticService;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectService;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectStatisticService;
import cn.com.qmth.examcloud.core.print.service.ProjectBackupSettingService;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectLessInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingprojectstatistic.PrintingProjectStatisticConvert;
import cn.com.qmth.examcloud.core.print.service.bean.printingprojectstatistic.PrintingProjectStatisticInfo;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

import static cn.com.qmth.examcloud.core.print.common.Constants.PRT_CODE_500;

/**
 * @author: fengdesheng
 * @since: 2018/10/26
 */
@Service
public class PrintingProjectStatisticServiceImpl implements PrintingProjectStatisticService {
    private static final Logger log = LoggerFactory.getLogger(PrintingProjectStatisticServiceImpl.class);
    @Autowired
    private ProjectStatisticRepository projectStatisticRepository;
    @Autowired
    private PrintingProjectService printingProjectService;
    @Autowired
    private CourseStatisticService courseStatisticService;
    @Autowired
    private ProjectBackupSettingService projectBackupSettingService;

    @Override
    public PrintingProjectStatisticInfo getPrintingProjectStatistic(Long orgId, Long examId) {
        Check.isNull(orgId, "学校ID不能为空！");
        Check.isNull(examId, "考试ID不能为空！");

        PrintingProject project = printingProjectService.getPrintingProjectByOrgIdAndExamId(orgId, examId);
        if (project == null) {
            throw new StatusException(PRT_CODE_500, "当前项目不存在！");
        }

        ProjectStatistic statistic = projectStatisticRepository.getProjectStatisticByProjectId(project.getId());
        return PrintingProjectStatisticConvert.of(project, statistic);
    }

    @Override
    public void refreshPrintingProjectStatistic(Long orgId, Long examId) {
        Check.isNull(orgId, "学校ID不能为空！");
        Check.isNull(examId, "考试ID不能为空！");

        PrintingProject project = printingProjectService.getPrintingProjectByOrgIdAndExamId(orgId, examId);
        if (project == null) {
            throw new StatusException(PRT_CODE_500, "当前项目不存在！");
        }

        this.doStatistic(Lists.newArrayList(new PrintingProjectLessInfo(orgId, examId, project.getId())));
    }

    @Override
    public void initAllPrintingProjectStatistic() {
        log.debug("initAllPrintingProjectStatistic...");

        //只处理已有的印刷项目
        List<PrintingProjectLessInfo> projects = printingProjectService.getPrintingProjectLessInfoList();

        this.doStatistic(projects);
    }

    private void doStatistic(List<PrintingProjectLessInfo> projects) {
        if (projects == null || projects.isEmpty()) {
            return;
        }

        for (PrintingProjectLessInfo project : projects) {
            ProjectStatistic statistic = projectStatisticRepository.getProjectStatisticByProjectId(project.getProjectId());
            if (statistic == null) {
                statistic = new ProjectStatistic();
                statistic.setProjectId(project.getProjectId());
            }
            //人科次
            int totalStudent = this.calculateTotalStudent(project.getOrgId(), project.getExamId());

            //课程数量
            int totalCourse = this.calculateTotalCourse(project.getOrgId(), project.getExamId());

            //试卷数量
            int totalPaper = this.calculateTotalPaper(project.getOrgId(), project.getExamId());

            //试卷袋数量
            int totalPkg = this.calculateTotalPkg(project.getOrgId(), project.getExamId());

            //常规-A3数量

            //常规-A4数量

            //projectBackupSettingService
            //备份-A3数量

            //备份-A4数量
            //Double ceil = Math.ceil(getNormalA4() / 2.0);
        }
    }

    private int calculateTotalStudent(Long orgId, Long examId) {
        return 0;
    }

    private int calculateTotalCourse(Long orgId, Long examId) {
        return 0;
    }

    private int calculateTotalPaper(Long orgId, Long examId) {
        return 0;
    }

    private int calculateTotalPkg(Long orgId, Long examId) {
        return 0;
    }

}